PROGRAM euler
  ! R. Abgrall, University of Zurich
  ! remi.abgrall@math.uzh.ch
  !
  ! Solution of the Euler equation by second order RD scheme
  !*********************************************************
  !Ref:M. Ricchiuto, R. Abgrall,
  ! Explicit Runge Kutta schemes for time dependent problems:
  ! second order case,
  ! J. Comput. Phys., 229(16), pp 5653-5691, 2010.
  !
  USE stroumpf
  IMPLICIT NONE
  INTEGER :: n, niter
  REAL, DIMENSION(:,:),ALLOCATABLE :: u, up,  un,  vn,s ,wp, sp, buff
  REAL, DIMENSION(:,:),ALLOCATABLE :: moy
  REAL, ALLOCATABLE,DIMENSION(:):: x
  ! u^n,u^{n}, u^{n+1}, moyenne Roe, contrib n, work_jacobi
  REAL, DIMENSION(:,:,:),ALLOCATABLE :: diag ! diagonale  matrice a inverser
  REAL, DIMENSION(:,:,:,:),ALLOCATABLE :: a! coeff extradiag (i-1,i+1)
  INTEGER, DIMENSION(:,:),ALLOCATABLE:: nubo
  REAL,ALLOCATABLE,DIMENSION(:):: rap
  REAL,DIMENSION(3):: U1,U2
  REAL :: dx,dt, cfl, eps, ub, deltax, deltat, xk1, xk2, xn, dtloc
  REAL :: cfloc,dtloc0,resin
  INTEGER :: k, j, iseg, i1, i2, kk, kite, iordre, kiter, ki, kiseg, restart,ischema
  REAL ::  t, moyy,  gamma, ro, v, p,tfin
  REAL, DIMENSION(2) :: beta
  INTEGER, DIMENSION(2,2):: nub
  INTEGER,DIMENSION(3) :: i
  CHARACTER(LEN = 10) :: name, name2
  gamma=1.4
  !
  ! initialisation
  !
  WRITE(*,*) 'N=?'
  READ(*,*) N
  WRITE(*,*) " ordre1=1, ordre2=2"
  READ(*,*) iordre
  WRITE(*,*) "Schema Roe=0,LxF=1"
  READ(*,*) ischema

  WRITE(*,*) " cfl"
  READ(*,*) cfl
  WRITE(*,*) " niter"
  READ(*,*) niter
  WRITE(*,*) "Restart=1, 0: pure initialisation"
  READ(*,*) restart
  WRITE(*,*) "Final time"
  READ(*,*) tfin
  ALLOCATE(u(3,0:n+1),up(3,0:n+1),un(3,0:n+1),vn(3,0:n+1),wp(3,0:n+1),rap(0:n+1))
  ALLOCATE(sp(3,0:n+1),buff(3,0:n+1),s(3,0:n+1))
  ALLOCATE(moy(3,1:n+1),diag(3,3,0:n+1),a(3,3,2,0:n+1))
  ALLOCATE(nubo(2,1:n),x(0:n+1))
  t=0
  dx=1./float(n+1)
  ! Construction of the mesh
  DO j=0,n+1
     x(j)=REAL(j)*dx
  ENDDO

  eps=1.e-6
  IF (restart==0) THEN
     un= init(x,gamma,n)!,u1,u2)
  ELSE
     READ(2) un
  ENDIF

  up=un
  u=un
  ! Geometry
  DO j=1,n !  ieme element [i-1,i]
     nubo(1,j)=j-1
     nubo(2,j)=j
  ENDDO

  !
  ! Loop over time
  !
  DO k=1,niter

     un=u
     up=u

     ! calculation of the time step
     dt=cfl* dx/vmax(n,gamma,u)
     dt=MIN(dt,tfin-t)
     IF (dt.LE.1.e-10) EXIT
     t=t+dt
     IF (MOD(k,100)==0   )  PRINT*, t,dt
     !
     ! first step of the "RK" method: evaluation of the residual
     call res(ischema,n,iordre,un,u,nubo,dx,dt,gamma, sp)
     sp(:,0)=0; sp(:,n+1)=0. ! boundary conditions
     up=un- sp/dx          ! the time step is taken into account in sp
! for second order in time: secon loop
     IF (iordre>1) THEN
        call res(ischema,n,iordre,up,u,nubo,dx,dt,gamma, sp)
        sp(:,0)=0; sp(:,n+1)=0. ! boundary conditions
        u=up-sp/dx          ! the time step is taken into account in sp
     ELSE
        u=up
     ENDIF

  ENDDO
  REWIND(1)
  open(1,file='ro_eul.dat',form="formatted")
  open(2,file='u_eul.dat',form="formatted")
  open(3,file='p_eul.dat',form="formatted")
  open(4,file='s_eul.dat',form="formatted")
  DO j=1,n
     ro=u(1,j)
     v=u(2,j)/ro
     p=(gamma-1)*(u(3,j)-0.5*ro*v*v)
     WRITE(1,10) x(j),ro
     WRITE(2,10) x(j),v
     WRITE(3,10) x(j),p
     WRITE(4,10) x(j), alog(p)-1.4*alog(ro)
  ENDDO
  WRITE(20) un
  10 format(5(e10.4,1x))
END PROGRAM Euler



